/* deckofcards.r */

/********************************************************************************/

!include <sculptor.h>
!include <http.h>
!include <www.h>

/********************************************************************************/
/*				Data															*/
/********************************************************************************/
. Here we define the structure of the http://deckofcardsapi.com/api response
!record response {
	deck_id,,a16,,v
	remaining,,i4
	shuffled,,i2
	success,,i2
	!record cards[1] {
		value,"Value",a10
		suit,"Suit",a10
		code,"Code",a2
	}
}

!temp numDraw,,i4
!temp numDecks,,i4
!temp shuffle,,i4

/********************************************************************************/
/*				Windows															*/
/********************************************************************************/

-window wintask at 364,226 {
	max_width = 847
	max_height = 498
	event = evWinTask
	style = WS_CLOSEBUTTON | WS_MAXBUTTON | WS_MINBUTTON
	data_font = "Arial@12"
	label_font = "Arial@12"
	title = "Deck of Cards REST client"

	+graphic graInfo at 210,30 {
		type = GT_RECTANGLE
		width = 603
		height = 90
	}

	+graphic graDraw at 25,255 {
		type = GT_RECTANGLE
		width = 160
		height = 170
	}

	+graphic graNew at 25,30 {
		type = GT_RECTANGLE
		width = 160
		height = 200
	}

	+graphic graCardImage at 526,133 {
		type = GT_IMAGE
		style = WS_INVISIBLE
	}

	+textbox txtDeckId at 232,64 {
		field = response.deck_id
		label = "Deck Id"
		rows = 1
		width = 100
		height = 30
		mode = EM_READONLY
	}

	+textbox txtRemaining at 385,64 {
		field = response.remaining
		label = "Remaining"
		rows = 1
		width = 100
		height = 30
		mode = EM_READONLY
	}

	+textbox txtShuffled at 538,64 {
		field = response.shuffled
		label = "Shuffled"
		rows = 1
		width = 100
		height = 30
		mode = EM_READONLY
	}

	+textbox txtSuccess at 691,64 {
		field = response.success
		label = "Success"
		rows = 1
		width = 100
		height = 30
		mode = EM_READONLY
	}

	+button btnNew at 55,50 {
		label = "&New"
		width = 100
		height = 50
		event = evNew
	}

	+textbox txtNumDecks at 55,140 {
		field = tmp.numDecks
		label = "N&um of decks"
		rows = 1
		width = 100
		height = 30
	}

	+button btnDraw at 55,275 {
		label = "&Draw"
		width = 100
		height = 50
		event = evDraw
	}

	+textbox txtNumDraw at 55,365 {
		field = tmp.numDraw
		label = "Nu&m to draw"
		rows = 1
		width = 100
		height = 30
	}

	checkbox group chkGrpShuffle {

		+button chkShuffle at 55,190 {
			label = "&Shuflle"
			field = tmp.shuffle
		}
	}

	+table tblCards at 210,160 {
		source_object = cards
		title = "Cards drawned"
		rows = 12
		title_font = "Arial,b@12"
		event_enable = EV_SELECT
		event = evTable
	}
}

/********************************************************************************/
/*				Definitions														*/
/********************************************************************************/

!define DECK_API_BASE	"http://deckofcardsapi.com/api/deck/"
	
/********************************************************************************/
/*				Code															*/
/********************************************************************************/

	fSetup()
	fMain()
	exit

/********************************************************************************/

!function fClearImage() {
	. Hide the graphic just in case there is an image being shown
	hide graCardImage

	. Unselect any table selected card
	set_selected(tblCards, 0, FALSE)
}

/*------------------------------------------------------------------------------*/

!function fInitialiseData() {
	clear response
	response.deck_id = "none"

	fClearImage()
}

/*------------------------------------------------------------------------------*/

!function fSetup() {

	fInitialiseData()

	. Default number of decks, draw number and shuffle options
	tmp.numDecks = 1
	tmp.shuffle = 1
	tmp.numDraw = 1

	. Set the cards subrecord to autoredim so that it will be resized to the returned size
	response.cards->flags = RECFL_AUTOREDIM

	. Create the window
	create wintask

	. Display data and enable/disable controls
	fDisplayResponse()
}

/*------------------------------------------------------------------------------*/

!function fMain() {

	dialog cancel=MAIN_FINISH

MAIN_FINISH:
	return
}

/*------------------------------------------------------------------------------*/

!function fCardValue(idx) {
	if (strlen(response.cards[idx].value) = 1) {
		. Two numbers between 2 and 9 (Substract '0' = 48)
		return asc(response.cards[idx].value) - 48
	}

	switch (response.cards[idx].value) {
	case = "10": return 10
	case = "JACK": return 11
	case = "QUEEN": return 12
	case = "KING": return 13
	case = "ACE": return 14
	}
}

/*------------------------------------------------------------------------------*/

!function fCardComparator(idx1, idx2) {
!temp ret,,i4

	. Compare suits
	ret = asc(response.cards[idx1].suit) - asc(response.cards[idx2].suit)
	if (ret <> 0) return ret

	. Compare values on same suit
	return fCardValue(idx2) - fCardValue(idx1)
}

/*------------------------------------------------------------------------------*/

!function fDisplayResponse() {
	if (response.deck_id = "none") {
		. Disable needed controls
		disable btnDraw, txtNumDraw, txtDeckId, txtRemaining, txtShuffled, txtSuccess
		wintask.tblCards->max_line = 0
	} else {
		. Sort the cards
		sort(response.cards, 0, fCardComparator)

		. Enable needed controls
		enable btnDraw, txtNumDraw, txtDeckId, txtRemaining, txtShuffled, txtSuccess

		. Set the number of table lines with information to the cards subredord dimension
		wintask.tblCards->max_line = dim(response.cards, 1)
	}

	. Show all the information
	display wintask
}

/*------------------------------------------------------------------------------*/

!function fShowImage(code) {
	show graCardImage
	graCardImage->image = "images/" + tolower(code) + ".jpg"
}

/*------------------------------------------------------------------------------*/

!function fSendQuery(url) {
!temp status,,i4

	status = http_simple_send(url, METHOD_GET, 0, NULL, response)

	. Check errors
	if (status = -1) {
		error "HTTP error: " + sys.HttpErrorMessage
		return FALSE
	}
	if (status <> SC_OK) {
		error "HTTP error (" + tostr(status) + "): " + sys.HttpResponseMessage
		return FALSE
	}
	
	return TRUE
}

/*------------------------------------------------------------------------------*/

!function fSendAndDisplay(url) {
	if (fSendQuery(url) = TRUE) fDisplayResponse()
}

/*------------------------------------------------------------------------------*/
/*			Deck of Cards REST API												*/
/*------------------------------------------------------------------------------*/

. URL: http://deckofcardsapi.com/api/deck/new[/shuffle]?deck_count=1

!function fQueryNew() {
!temp url,,a255,,v

	url = DECK_API_BASE + "new"

	if (tmp.shuffle <> 0) url = url + "/shuffle"

	fSendAndDisplay(url + "?deck_count="+tostr(tmp.numDecks))
}

/*------------------------------------------------------------------------------*/

. URL: http://deckofcardsapi.com/api/deck/<<deck_id>>/draw/?count=2

!function fQueryDraw() {
	fSendAndDisplay(DECK_API_BASE + response.deck_id + "/draw?count=" + tostr(tmp.numDraw))
}

/*------------------------------------------------------------------------------*/
/*			Event functions														*/
/*------------------------------------------------------------------------------*/

!function evNew(EventCode) {
	if (EventCode != EV_BUTTON_CLICKED) return

	fInitialiseData()
	fQueryNew()
}

/*------------------------------------------------------------------------------*/

!function evDraw(EventCode) {
	if (EventCode != EV_BUTTON_CLICKED) return
	fClearImage()
	fQueryDraw()
}

/*------------------------------------------------------------------------------*/

!function evTable(EventCode, tableId, line) {
	if (EventCode != EV_SELECT) return

	. Show the selected image
	fShowImage(response.cards[line].code)
}

/*------------------------------------------------------------------------------*/

!function evWinTask(EventCode) {

	switch (EventCode) {
	case == EV_CLOSE:
		exit
	}
}

/********************************************************************************/
!chargrid OFF
/********************************************************************************/
